from abc import ABC, abstractmethod
import sys
sys.path.append('C:/Users/HP/OneDrive/Praxisprojekt Semester 6/praxisprojekt-technologiebasierte-innovationen')
from Project_tests.converter import DanisConverter as DaniConv

class IRomanNumerals(ABC):
    @abstractmethod
    def to_roman(self, num):
        pass

class RomanNumerals(IRomanNumerals):
    def to_roman(self, num):
        if not isinstance(num, int):
            raise ValueError("Eingabe muss eine ganze Zahl sein")
        if num <= 0 or num >= 4000:
            raise ValueError("Zahl muss zwischen 1 und 3999 liegen")

        val = [
            (1000, "M"), (900, "CM"), (500, "D"), (400, "CD"),
            (100, "C"), (90, "XC"), (50, "L"), (40, "XL"),
            (10, "X"), (9, "IX"), (5, "V"), (4, "IV"),
            (1, "I")
        ]

        result = ""
        for (value, numeral) in val:
            while num >= value:
                result += numeral
                num -= value
        return result


import unittest


class TestRomanNumerals(unittest.TestCase):
    def setUp(self):
        self.converter = DaniConv()
        self.converter.to_roman = self.converter.convert

    def test_to_roman_basic(self):
        self.assertEqual(self.converter.to_roman(1), "I")
        self.assertEqual(self.converter.to_roman(5), "V")
        self.assertEqual(self.converter.to_roman(10), "X")
        self.assertEqual(self.converter.to_roman(50), "L")
        self.assertEqual(self.converter.to_roman(100), "C")
        self.assertEqual(self.converter.to_roman(500), "D")
        self.assertEqual(self.converter.to_roman(1000), "M")

    def test_to_roman_subtraction(self):
        self.assertEqual(self.converter.to_roman(4), "IV")
        self.assertEqual(self.converter.to_roman(9), "IX")
        self.assertEqual(self.converter.to_roman(40), "XL")
        self.assertEqual(self.converter.to_roman(90), "XC")
        self.assertEqual(self.converter.to_roman(400), "CD")
        self.assertEqual(self.converter.to_roman(900), "CM")

    def test_to_roman_addition(self):
        self.assertEqual(self.converter.to_roman(2), "II")
        self.assertEqual(self.converter.to_roman(3), "III")
        self.assertEqual(self.converter.to_roman(6), "VI")
        self.assertEqual(self.converter.to_roman(8), "VIII")
        self.assertEqual(self.converter.to_roman(30), "XXX")
        self.assertEqual(self.converter.to_roman(80), "LXXX")

    def test_to_roman_mixed(self):
        self.assertEqual(self.converter.to_roman(14), "XIV")
        self.assertEqual(self.converter.to_roman(19), "XIX")
        self.assertEqual(self.converter.to_roman(29), "XXIX")
        self.assertEqual(self.converter.to_roman(44), "XLIV")
        self.assertEqual(self.converter.to_roman(99), "XCIX")
        self.assertEqual(self.converter.to_roman(444), "CDXLIV")
        self.assertEqual(self.converter.to_roman(999), "CMXCIX")

    def test_to_roman_large_numbers(self):
        self.assertEqual(self.converter.to_roman(1001), "MI")
        self.assertEqual(self.converter.to_roman(1987), "MCMLXXXVII")
        self.assertEqual(self.converter.to_roman(2023), "MMXXIII")
        self.assertEqual(self.converter.to_roman(3999), "MMMCMXCIX")

    def test_to_roman_invalid(self):
        with self.assertRaises(ValueError):
            self.converter.to_roman(0)
        with self.assertRaises(ValueError):
            self.converter.to_roman(-1)
        with self.assertRaises(ValueError):
            self.converter.to_roman(4000)
        with self.assertRaises(ValueError):
            self.converter.to_roman("ABC")
        with self.assertRaises(ValueError):
            self.converter.to_roman(None)
        with self.assertRaises(ValueError):
            self.converter.to_roman(3.14)

if __name__ == "__main__":
    unittest.main()